searchData={"items":[{"type":"module","title":"Plug.Cowboy","doc":"Adapter interface to the Cowboy2 webserver.","ref":"Plug.Cowboy.html"},{"type":"module","title":"Options - Plug.Cowboy","doc":"* `:net` - If using `:inet` (IPv4 only - the default) or `:inet6` (IPv6)\n\n  * `:ip` - the ip to bind the server to.\n    Must be either a tuple in the format `{a, b, c, d}` with each value in `0..255` for IPv4,\n    or a tuple in the format `{a, b, c, d, e, f, g, h}` with each value in `0..65535` for IPv6,\n    or a tuple in the format `{:local, path}` for a unix socket at the given `path`.\n    If you set an IPv6, the `:net` option will be automatically set to `:inet6`.\n    If both `:net` and `:ip` options are given, make sure they are compatible\n    (i.e. give a IPv4 for `:inet` and IPv6 for `:inet6`).\n    Also, see \"Loopback vs Public IP Addresses\".\n\n  * `:port` - the port to run the server.\n    Defaults to 4000 (http) and 4040 (https).\n    Must be 0 when `:ip` is a `{:local, path}` tuple.\n\n  * `:dispatch` - manually configure Cowboy's dispatch.\n    If this option is used, the given plug won't be initialized\n    nor dispatched to (and doing so becomes the user's responsibility).\n\n  * `:ref` - the reference name to be used.\n    Defaults to `plug.HTTP` (http) and `plug.HTTPS` (https).\n    Note, the default reference name does not contain the port so in order\n    to serve the same plug on multiple ports you need to set the `:ref` accordingly,\n    e.g.: `ref: MyPlug_HTTP_4000`, `ref: MyPlug_HTTP_4001`, etc.\n    This is the value that needs to be given on shutdown.\n\n  * `:compress` - Cowboy will attempt to compress the response body.\n    Defaults to false.\n\n  * `:stream_handlers` - List of Cowboy `stream_handlers`,\n    see [Cowboy docs](https://ninenines.eu/docs/en/cowboy/2.5/manual/cowboy_http/).\n\n  * `:protocol_options` - Specifies remaining protocol options,\n    see [Cowboy docs](https://ninenines.eu/docs/en/cowboy/2.5/manual/cowboy_http/).\n\n  * `:transport_options` - A keyword list specifying transport options,\n    see [Ranch docs](https://ninenines.eu/docs/en/ranch/1.7/manual/ranch/).\n    By default `:num_acceptors` will be set to `100` and `:max_connections`\n    to `16_384`.\n\nAll other options given at the top level must configure the underlying\nsocket. For HTTP connections, those options are listed under\n[`ranch_tcp`](https://ninenines.eu/docs/en/ranch/1.7/manual/ranch_tcp/).\nFor example, you can set `:ipv6_v6only` to true if you want to bind only\non IPv6 addresses.\n\nFor HTTPS (SSL) connections, those options are described in\n[`ranch_ssl`](https://ninenines.eu/docs/en/ranch/1.7/manual/ranch_ssl/).\nSee `https/3` for an example and read `Plug.SSL.configure/1` to\nunderstand about our SSL defaults.\n\nWhen using a Unix socket, OTP 21+ is required for `Plug.Static` and\n`Plug.Conn.send_file/3` to behave correctly.","ref":"Plug.Cowboy.html#module-options"},{"type":"module","title":"Safety limits - Plug.Cowboy","doc":"Cowboy sets different limits on URL size, header length, number of\nheaders and so on to protect your application from attacks. For example,\nthe request line length defaults to 10k, which means Cowboy will return\n414 if a larger URL is given. You can change this under `:protocol_options`:\n\n    protocol_options: [max_request_line_length: 50_000]\n\nKeep in mind though increasing those limits can pose a security risk.\nOther times, browsers and proxies along the way may have equally strict\nlimits, which means the request will still fail or the URL will be\npruned. You can [consult all limits here](https://ninenines.eu/docs/en/cowboy/2.5/manual/cowboy_http/).","ref":"Plug.Cowboy.html#module-safety-limits"},{"type":"module","title":"Loopback vs Public IP Addresses - Plug.Cowboy","doc":"Should your application bind to a loopback address, such as `::1` (IPv6) or\n`127.0.0.1` (IPv4), or a public one, such as `::0` (IPv6) or `0.0.0.0`\n(IPv4)? It depends on how (and whether) you want it to be reachable from\nother machines.\n\nLoopback addresses are only reachable from the same host (`localhost` is\nusually configured to resolve to a loopback address). You may wish to use one if:\n\n- Your app is running in a development environment (such as your laptop) and\nyou don't want others on the same network to access it.\n- Your app is running in production, but behind a reverse proxy. For example,\nyou might have Nginx bound to a public address and serving HTTPS, but\nforwarding the traffic to your application running on the same host. In that\ncase, having your app bind to the loopback address means that Nginx can reach\nit, but outside traffic can only reach it via Nginx.\n\nPublic addresses are reachable from other hosts. You may wish to use one if:\n\n- Your app is running in a container. In this case, its loopback address is\nreachable only from within the container; to be accessible from outside the\ncontainer, it needs to bind to a public IP address.\n- Your app is running in production without a reverse proxy, using Cowboy's\nSSL support.","ref":"Plug.Cowboy.html#module-loopback-vs-public-ip-addresses"},{"type":"module","title":"Logging - Plug.Cowboy","doc":"You can configure which exceptions are logged via `:log_exceptions_with_status_code`\napplication environment variable. If the status code returned by `Plug.Exception.status/1`\nfor the exception falls into any of the configured ranges, the exception is logged.\nBy default it's set to `[500..599]`.\n\n    config :plug_cowboy,\n      log_exceptions_with_status_code: [400..599]\n\nBy default, `Plug.Cowboy` includes the entire `conn` to the log metadata for exceptions.\nHowever, this metadata may contain sensitive information such as security headers or \ncookies, which may be logged in plain text by certain logging backends. To prevent this,\nyou can configure the `:conn_in_exception_metadata` option to not include the `conn` in the metadata.\n\n    config :plug_cowboy,\n      conn_in_exception_metadata: false","ref":"Plug.Cowboy.html#module-logging"},{"type":"module","title":"Instrumentation - Plug.Cowboy","doc":"Plug.Cowboy uses the `:telemetry` library for instrumentation. The following\nspan events are published during each request:\n\n  * `[:cowboy, :request, :start]` - dispatched at the beginning of the request\n  * `[:cowboy, :request, :stop]` - dispatched at the end of the request\n  * `[:cowboy, :request, :exception]` - dispatched at the end of a request that exits\n\nA single event is published when the request ends with an early error:\n  * `[:cowboy, :request, :early_error]` - dispatched for requests terminated early by Cowboy\n\nSee [`cowboy_telemetry`](https://github.com/beam-telemetry/cowboy_telemetry#telemetry-events)\nfor more details on the events.\n\nTo opt-out of this default instrumentation, you can manually configure\ncowboy with the option `stream_handlers: [:cowboy_stream_h]`.","ref":"Plug.Cowboy.html#module-instrumentation"},{"type":"module","title":"WebSocket support - Plug.Cowboy","doc":"Plug.Cowboy supports upgrading HTTP requests to WebSocket connections via \nthe use of the `Plug.Conn.upgrade_adapter/3` function, called with `:websocket` as the second\nargument. Applications should validate that the connection represents a valid WebSocket request\nbefore calling this function (Cowboy will validate the connection as part of the upgrade\nprocess, but does not provide any capacity for an application to be notified if the upgrade is\nnot successful). If an application wishes to negotiate WebSocket subprotocols or otherwise set\nany response headers, it should do so before calling `Plug.Conn.upgrade_adapter/3`.\n\nThe third argument to `Plug.Conn.upgrade_adapter/3` defines the details of how Plug.Cowboy\nshould handle the WebSocket connection, and must take the form `{handler, handler_opts,\nconnection_opts}`, where values are as follows:\n\n* `handler` is a module which implements the\n  [`:cowboy_websocket`](https://ninenines.eu/docs/en/cowboy/2.6/manual/cowboy_websocket/)\n  behaviour. Note that this module will NOT have its `c:cowboy_websocket.init/2` callback\n  called; only the 'later' parts of the `:cowboy_websocket` lifecycle are supported\n* `handler_opts` is an arbitrary term which will be passed as the argument to\n  `c:cowboy_websocket.websocket_init/1`\n* `connection_opts` is a map with any of [Cowboy's websockets options](https://ninenines.eu/docs/en/cowboy/2.6/manual/cowboy_websocket/#_opts)","ref":"Plug.Cowboy.html#module-websocket-support"},{"type":"function","title":"Plug.Cowboy.child_spec/1","doc":"A function for starting a Cowboy2 server under Elixir v1.5+ supervisors.\n\nIt supports all options as specified in the module documentation plus it\nrequires the following two options:\n\n  * `:scheme` - either `:http` or `:https`\n  * `:plug` - such as `MyPlug` or `{MyPlug, plug_opts}`","ref":"Plug.Cowboy.html#child_spec/1"},{"type":"function","title":"Examples - Plug.Cowboy.child_spec/1","doc":"Assuming your Plug module is named `MyApp` you can add it to your\nsupervision tree by using this function:\n\n    children = [\n      {Plug.Cowboy, scheme: :http, plug: MyApp, options: [port: 4040]}\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one)","ref":"Plug.Cowboy.html#child_spec/1-examples"},{"type":"function","title":"Plug.Cowboy.http/3","doc":"Runs cowboy under http.","ref":"Plug.Cowboy.html#http/3"},{"type":"function","title":"Example - Plug.Cowboy.http/3","doc":"# Starts a new interface\n    Plug.Cowboy.http MyPlug, [], port: 80\n\n    # The interface above can be shutdown with\n    Plug.Cowboy.shutdown MyPlug.HTTP","ref":"Plug.Cowboy.html#http/3-example"},{"type":"function","title":"Plug.Cowboy.https/3","doc":"Runs cowboy under https.\n\nBesides the options described in the module documentation,\nthis function sets defaults and accepts all options defined\nin `Plug.SSL.configure/1`.","ref":"Plug.Cowboy.html#https/3"},{"type":"function","title":"Example - Plug.Cowboy.https/3","doc":"# Starts a new interface\n    Plug.Cowboy.https MyPlug, [],\n      port: 443,\n      password: \"SECRET\",\n      otp_app: :my_app,\n      keyfile: \"priv/ssl/key.pem\",\n      certfile: \"priv/ssl/cert.pem\",\n      dhfile: \"priv/ssl/dhparam.pem\"\n\n    # The interface above can be shutdown with\n    Plug.Cowboy.shutdown MyPlug.HTTPS","ref":"Plug.Cowboy.html#https/3-example"},{"type":"function","title":"Plug.Cowboy.shutdown/1","doc":"Shutdowns the given reference.","ref":"Plug.Cowboy.html#shutdown/1"},{"type":"module","title":"Plug.Cowboy.Drainer","doc":"Process to drain cowboy connections at shutdown.\n\nWhen starting `Plug.Cowboy` in a supervision tree, it will create a listener that receives\nrequests and creates a connection process to handle that request. During shutdown, a\n`Plug.Cowboy` process will immediately exit, closing the listener and any open connections\nthat are still being served. However, in most cases, it is desirable to allow connections\nto complete before shutting down.\n\nThis module provides a process that during shutdown will close listeners and wait\nfor connections to complete. It should be placed after other supervised processes that\nhandle cowboy connections.","ref":"Plug.Cowboy.Drainer.html"},{"type":"module","title":"Options - Plug.Cowboy.Drainer","doc":"The following options can be given to the child spec:\n\n  * `:refs` - A list of refs to drain. `:all` is also supported and will drain all cowboy\n    listeners, including those started by means other than `Plug.Cowboy`.\n\n  * `:id` - The ID for the process.\n    Defaults to `Plug.Cowboy.Drainer`.\n\n  * `:shutdown` - How long to wait for connections to drain.\n    Defaults to 5000ms.\n\n  * `:check_interval` - How frequently to check if a listener's\n    connections have been drained. Defaults to 1000ms.","ref":"Plug.Cowboy.Drainer.html#module-options"},{"type":"module","title":"Examples - Plug.Cowboy.Drainer","doc":"# In your application\n    def start(_type, _args) do\n      children = [\n        {Plug.Cowboy, scheme: :http, plug: MyApp, options: [port: 4040]},\n        {Plug.Cowboy, scheme: :https, plug: MyApp, options: [port: 4041]},\n        {Plug.Cowboy.Drainer, refs: [MyApp.HTTP, MyApp.HTTPS]}\n      ]\n\n      opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n      Supervisor.start_link(children, opts)\n    end","ref":"Plug.Cowboy.Drainer.html#module-examples"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v2.7.1 - Changelog","doc":"#","ref":"changelog.html#v2-7-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support Cowboy 2.11","ref":"changelog.html#enhancements"},{"type":"extras","title":"v2.7.0 - Changelog","doc":"#","ref":"changelog.html#v2-7-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Do not allow Cowboy 2.11 due to backwards incompatible changes","ref":"changelog.html#enhancements"},{"type":"extras","title":"v2.6.2 - Changelog","doc":"#","ref":"changelog.html#v2-6-2"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Fix warnings on Elixir v1.15+","ref":"changelog.html#enhancements"},{"type":"extras","title":"v2.6.1 - Changelog","doc":"#","ref":"changelog.html#v2-6-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Allow for opt-out of conn metadata on exception logs\n * Support `:check_interval` in drainer (in addition to `:drain_check_interval`)","ref":"changelog.html#enhancements"},{"type":"extras","title":"v2.6.0 - Changelog","doc":"#","ref":"changelog.html#v2-6-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support websocket upgrades\n  * Require Plug v1.14+ and Elixir v1.10+","ref":"changelog.html#enhancements"},{"type":"extras","title":"v2.5.2 - Changelog","doc":"#","ref":"changelog.html#v2-5-2"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Fix warnings when running on telemetry 1.x","ref":"changelog.html#enhancements"},{"type":"extras","title":"v2.5.1 - Changelog","doc":"#","ref":"changelog.html#v2-5-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Allow to configure which errors should be logged\n  * Support telemetry 0.4.x or 1.x","ref":"changelog.html#enhancements"},{"type":"extras","title":"v2.5.0 - Changelog","doc":"#","ref":"changelog.html#v2-5-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Return `:conn` as Logger metadata on translator\n  * Support Ranch 2.0\n  * Support the `:net` option so developers can work with keyword lists\n  * Remove previously deprecated options","ref":"changelog.html#enhancements"},{"type":"extras","title":"v2.4.1 (2020-10-31) - Changelog","doc":"#","ref":"changelog.html#v2-4-1-2020-10-31"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Properly format linked exits","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.4.0 (2020-10-11) - Changelog","doc":"#","ref":"changelog.html#v2-4-0-2020-10-11"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Add [cowboy_telemetry](https://github.com/beam-telemetry/cowboy_telemetry/) as a dependency and enable it by default","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.3.0 (2020-06-11) - Changelog","doc":"Plug.Cowboy requires Elixir v1.7 or later.\n\n#","ref":"changelog.html#v2-3-0-2020-06-11"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* The telemetry events added in version v2.2.0 does not work as expected. The whole v2.2.x branch has been retired in favor of v2.3.0.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.2.2 (2020-05-25) - Changelog","doc":"#","ref":"changelog.html#v2-2-2-2020-05-25"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Emit telemetry event for Cowboy early errors\n  * Improve error messages for Cowboy early errors","ref":"changelog.html#enhancements"},{"type":"extras","title":"v2.2.1 (2020-04-21) - Changelog","doc":"#","ref":"changelog.html#v2-2-1-2020-04-21"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Use proper telemetry metadata for exceptions","ref":"changelog.html#enhancements"},{"type":"extras","title":"v2.2.0 (2020-04-21) - Changelog","doc":"#","ref":"changelog.html#v2-2-0-2020-04-21"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Include telemetry support","ref":"changelog.html#enhancements"},{"type":"extras","title":"v2.1.3 (2020-04-14) - Changelog","doc":"#","ref":"changelog.html#v2-1-3-2020-04-14"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Properly support the :options option before removal","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.1.2 (2020-01-28) - Changelog","doc":"#","ref":"changelog.html#v2-1-2-2020-01-28"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Properly deprecate the :timeout option before removal","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.1.1 (2020-01-08) - Changelog","doc":"#","ref":"changelog.html#v2-1-1-2020-01-08"},{"type":"extras","title":"Enhancement - Changelog","doc":"* Improve docs and simplify child spec API","ref":"changelog.html#enhancement"},{"type":"extras","title":"v2.1.0 (2019-06-27) - Changelog","doc":"#","ref":"changelog.html#v2-1-0-2019-06-27"},{"type":"extras","title":"Enhancement - Changelog","doc":"* Add `Plug.Cowboy.Drainer` for connection draining","ref":"changelog.html#enhancement"},{"type":"extras","title":"v2.0.2 (2019-03-18) - Changelog","doc":"#","ref":"changelog.html#v2-0-2-2019-03-18"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Unwrap `Plug.Conn.WrapperError` on handler error\n  * Include `crash_reason` as logger metadata","ref":"changelog.html#enhancements"},{"type":"extras","title":"v2.0.1 (2018-12-13) - Changelog","doc":"#","ref":"changelog.html#v2-0-1-2018-12-13"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Respect `:read_length` and `:read_timeout` in `read_body` with Cowboy 2","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.0.0 (2018-10-20) - Changelog","doc":"Extract `Plug.Adapters.Cowboy2` from Plug into `Plug.Cowboy`","ref":"changelog.html#v2-0-0-2018-10-20"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}